
! Copyright 2014 Fabio Cascioli, Jonas Lindert, Philipp Maierhoefer, Stefano Pozzorini
!
! This file is part of OpenLoops.
!
! OpenLoops is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.
!
! OpenLoops is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with OpenLoops.  If not, see <http://www.gnu.org/licenses/>.


module ol_loop_momentum_/**/REALKIND
  implicit none
  contains

!*********************************************
subroutine loop_mom_tens(Qloop,rank,Qarray)
! Qloop  = real Lorentz 4-momentum !! contravariant !!
! rank   = highest rank in the loop
! Qarray = array containing all independent tensor components up to highest rank
  use KIND_TYPES, only: REALKIND
  use ol_kinematics_/**/REALKIND, only: Std2LC_cmplx
  implicit none
  complex(REALKIND), intent(in)  :: Qloop(0:3)
  integer,           intent(in)  :: rank
  complex(REALKIND), intent(out) :: Qarray(:)
  complex(REALKIND) :: QloopLC(4)

  call Std2LC_cmplx(Qloop, QloopLC)

  select case (rank)
    case (0)
    call loop_mom_tens_0(QloopLC,Qarray)

    case (1)
    call loop_mom_tens_1(QloopLC,Qarray)

    case (2)
    call loop_mom_tens_2(QloopLC,Qarray)

    case (3)
    call loop_mom_tens_3(QloopLC,Qarray)

    case (4)
    call loop_mom_tens_4(QloopLC,Qarray)

    case (5)
    call loop_mom_tens_5(QloopLC,Qarray)

    case (6)
    call loop_mom_tens_6(QloopLC,Qarray)
  end select
end subroutine loop_mom_tens


!*********************************************
subroutine loop_mom_tens_0(QloopLC,Qarray)
  use KIND_TYPES, only: REALKIND
  implicit none
  complex(REALKIND), intent(in)  :: QloopLC(4)
  complex(REALKIND), intent(out) :: Qarray(1)

  Qarray(1) = 1

end subroutine loop_mom_tens_0


!*********************************************
subroutine loop_mom_tens_1(QloopLC,Qarray)
  use KIND_TYPES, only: REALKIND
  implicit none
  complex(REALKIND), intent(in)  :: QloopLC(4)
  complex(REALKIND), intent(out) :: Qarray(5)

  Qarray(1) = 1
  Qarray(2) = QloopLC(1)*Qarray(1)
  Qarray(3) = QloopLC(2)*Qarray(1)
  Qarray(4) = QloopLC(3)*Qarray(1)
  Qarray(5) = QloopLC(4)*Qarray(1)

end subroutine loop_mom_tens_1


!*********************************************
subroutine loop_mom_tens_2(QloopLC,Qarray)
  use KIND_TYPES, only: REALKIND
  implicit none
  complex(REALKIND), intent(in)  :: QloopLC(4)
  complex(REALKIND), intent(out) :: Qarray(15)

  Qarray(1) = 1
  Qarray(2) = QloopLC(1)*Qarray(1)
  Qarray(3) = QloopLC(2)*Qarray(1)
  Qarray(4) = QloopLC(3)*Qarray(1)
  Qarray(5) = QloopLC(4)*Qarray(1)
  Qarray(6) = QloopLC(1)*Qarray(2)
  Qarray(7) = QloopLC(2)*Qarray(2)
  Qarray(8) = QloopLC(3)*Qarray(2)
  Qarray(9) = QloopLC(4)*Qarray(2)
  Qarray(10) = QloopLC(2)*Qarray(3)
  Qarray(11) = QloopLC(3)*Qarray(3)
  Qarray(12) = QloopLC(4)*Qarray(3)
  Qarray(13) = QloopLC(3)*Qarray(4)
  Qarray(14) = QloopLC(4)*Qarray(4)
  Qarray(15) = QloopLC(4)*Qarray(5)

end subroutine loop_mom_tens_2


!*********************************************
subroutine loop_mom_tens_3(QloopLC,Qarray)
  use KIND_TYPES, only: REALKIND
  implicit none
  complex(REALKIND), intent(in)  :: QloopLC(4)
  complex(REALKIND), intent(out) :: Qarray(35)

  Qarray(1) = 1
  Qarray(2) = QloopLC(1)*Qarray(1)
  Qarray(3) = QloopLC(2)*Qarray(1)
  Qarray(4) = QloopLC(3)*Qarray(1)
  Qarray(5) = QloopLC(4)*Qarray(1)
  Qarray(6) = QloopLC(1)*Qarray(2)
  Qarray(7) = QloopLC(2)*Qarray(2)
  Qarray(8) = QloopLC(3)*Qarray(2)
  Qarray(9) = QloopLC(4)*Qarray(2)
  Qarray(10) = QloopLC(2)*Qarray(3)
  Qarray(11) = QloopLC(3)*Qarray(3)
  Qarray(12) = QloopLC(4)*Qarray(3)
  Qarray(13) = QloopLC(3)*Qarray(4)
  Qarray(14) = QloopLC(4)*Qarray(4)
  Qarray(15) = QloopLC(4)*Qarray(5)
  Qarray(16) = QloopLC(1)*Qarray(6)
  Qarray(17) = QloopLC(2)*Qarray(6)
  Qarray(18) = QloopLC(3)*Qarray(6)
  Qarray(19) = QloopLC(4)*Qarray(6)
  Qarray(20) = QloopLC(2)*Qarray(7)
  Qarray(21) = QloopLC(3)*Qarray(7)
  Qarray(22) = QloopLC(4)*Qarray(7)
  Qarray(23) = QloopLC(3)*Qarray(8)
  Qarray(24) = QloopLC(4)*Qarray(8)
  Qarray(25) = QloopLC(4)*Qarray(9)
  Qarray(26) = QloopLC(2)*Qarray(10)
  Qarray(27) = QloopLC(3)*Qarray(10)
  Qarray(28) = QloopLC(4)*Qarray(10)
  Qarray(29) = QloopLC(3)*Qarray(11)
  Qarray(30) = QloopLC(4)*Qarray(11)
  Qarray(31) = QloopLC(4)*Qarray(12)
  Qarray(32) = QloopLC(3)*Qarray(13)
  Qarray(33) = QloopLC(4)*Qarray(13)
  Qarray(34) = QloopLC(4)*Qarray(14)
  Qarray(35) = QloopLC(4)*Qarray(15)

end subroutine loop_mom_tens_3


!*********************************************
subroutine loop_mom_tens_4(QloopLC,Qarray)
  use KIND_TYPES, only: REALKIND
  implicit none
  complex(REALKIND), intent(in)  :: QloopLC(4)
  complex(REALKIND), intent(out) :: Qarray(70)

  Qarray(1) = 1
  Qarray(2) = QloopLC(1)*Qarray(1)
  Qarray(3) = QloopLC(2)*Qarray(1)
  Qarray(4) = QloopLC(3)*Qarray(1)
  Qarray(5) = QloopLC(4)*Qarray(1)
  Qarray(6) = QloopLC(1)*Qarray(2)
  Qarray(7) = QloopLC(2)*Qarray(2)
  Qarray(8) = QloopLC(3)*Qarray(2)
  Qarray(9) = QloopLC(4)*Qarray(2)
  Qarray(10) = QloopLC(2)*Qarray(3)
  Qarray(11) = QloopLC(3)*Qarray(3)
  Qarray(12) = QloopLC(4)*Qarray(3)
  Qarray(13) = QloopLC(3)*Qarray(4)
  Qarray(14) = QloopLC(4)*Qarray(4)
  Qarray(15) = QloopLC(4)*Qarray(5)
  Qarray(16) = QloopLC(1)*Qarray(6)
  Qarray(17) = QloopLC(2)*Qarray(6)
  Qarray(18) = QloopLC(3)*Qarray(6)
  Qarray(19) = QloopLC(4)*Qarray(6)
  Qarray(20) = QloopLC(2)*Qarray(7)
  Qarray(21) = QloopLC(3)*Qarray(7)
  Qarray(22) = QloopLC(4)*Qarray(7)
  Qarray(23) = QloopLC(3)*Qarray(8)
  Qarray(24) = QloopLC(4)*Qarray(8)
  Qarray(25) = QloopLC(4)*Qarray(9)
  Qarray(26) = QloopLC(2)*Qarray(10)
  Qarray(27) = QloopLC(3)*Qarray(10)
  Qarray(28) = QloopLC(4)*Qarray(10)
  Qarray(29) = QloopLC(3)*Qarray(11)
  Qarray(30) = QloopLC(4)*Qarray(11)
  Qarray(31) = QloopLC(4)*Qarray(12)
  Qarray(32) = QloopLC(3)*Qarray(13)
  Qarray(33) = QloopLC(4)*Qarray(13)
  Qarray(34) = QloopLC(4)*Qarray(14)
  Qarray(35) = QloopLC(4)*Qarray(15)
  Qarray(36) = QloopLC(1)*Qarray(16)
  Qarray(37) = QloopLC(2)*Qarray(16)
  Qarray(38) = QloopLC(3)*Qarray(16)
  Qarray(39) = QloopLC(4)*Qarray(16)
  Qarray(40) = QloopLC(2)*Qarray(17)
  Qarray(41) = QloopLC(3)*Qarray(17)
  Qarray(42) = QloopLC(4)*Qarray(17)
  Qarray(43) = QloopLC(3)*Qarray(18)
  Qarray(44) = QloopLC(4)*Qarray(18)
  Qarray(45) = QloopLC(4)*Qarray(19)
  Qarray(46) = QloopLC(2)*Qarray(20)
  Qarray(47) = QloopLC(3)*Qarray(20)
  Qarray(48) = QloopLC(4)*Qarray(20)
  Qarray(49) = QloopLC(3)*Qarray(21)
  Qarray(50) = QloopLC(4)*Qarray(21)
  Qarray(51) = QloopLC(4)*Qarray(22)
  Qarray(52) = QloopLC(3)*Qarray(23)
  Qarray(53) = QloopLC(4)*Qarray(23)
  Qarray(54) = QloopLC(4)*Qarray(24)
  Qarray(55) = QloopLC(4)*Qarray(25)
  Qarray(56) = QloopLC(2)*Qarray(26)
  Qarray(57) = QloopLC(3)*Qarray(26)
  Qarray(58) = QloopLC(4)*Qarray(26)
  Qarray(59) = QloopLC(3)*Qarray(27)
  Qarray(60) = QloopLC(4)*Qarray(27)
  Qarray(61) = QloopLC(4)*Qarray(28)
  Qarray(62) = QloopLC(3)*Qarray(29)
  Qarray(63) = QloopLC(4)*Qarray(29)
  Qarray(64) = QloopLC(4)*Qarray(30)
  Qarray(65) = QloopLC(4)*Qarray(31)
  Qarray(66) = QloopLC(3)*Qarray(32)
  Qarray(67) = QloopLC(4)*Qarray(32)
  Qarray(68) = QloopLC(4)*Qarray(33)
  Qarray(69) = QloopLC(4)*Qarray(34)
  Qarray(70) = QloopLC(4)*Qarray(35)

end subroutine loop_mom_tens_4


!*********************************************
subroutine loop_mom_tens_5(QloopLC,Qarray)
  use KIND_TYPES, only: REALKIND
  implicit none
  complex(REALKIND), intent(in)  :: QloopLC(4)
  complex(REALKIND), intent(out) :: Qarray(126)

  Qarray(1) = 1
  Qarray(2) = QloopLC(1)*Qarray(1)
  Qarray(3) = QloopLC(2)*Qarray(1)
  Qarray(4) = QloopLC(3)*Qarray(1)
  Qarray(5) = QloopLC(4)*Qarray(1)
  Qarray(6) = QloopLC(1)*Qarray(2)
  Qarray(7) = QloopLC(2)*Qarray(2)
  Qarray(8) = QloopLC(3)*Qarray(2)
  Qarray(9) = QloopLC(4)*Qarray(2)
  Qarray(10) = QloopLC(2)*Qarray(3)
  Qarray(11) = QloopLC(3)*Qarray(3)
  Qarray(12) = QloopLC(4)*Qarray(3)
  Qarray(13) = QloopLC(3)*Qarray(4)
  Qarray(14) = QloopLC(4)*Qarray(4)
  Qarray(15) = QloopLC(4)*Qarray(5)
  Qarray(16) = QloopLC(1)*Qarray(6)
  Qarray(17) = QloopLC(2)*Qarray(6)
  Qarray(18) = QloopLC(3)*Qarray(6)
  Qarray(19) = QloopLC(4)*Qarray(6)
  Qarray(20) = QloopLC(2)*Qarray(7)
  Qarray(21) = QloopLC(3)*Qarray(7)
  Qarray(22) = QloopLC(4)*Qarray(7)
  Qarray(23) = QloopLC(3)*Qarray(8)
  Qarray(24) = QloopLC(4)*Qarray(8)
  Qarray(25) = QloopLC(4)*Qarray(9)
  Qarray(26) = QloopLC(2)*Qarray(10)
  Qarray(27) = QloopLC(3)*Qarray(10)
  Qarray(28) = QloopLC(4)*Qarray(10)
  Qarray(29) = QloopLC(3)*Qarray(11)
  Qarray(30) = QloopLC(4)*Qarray(11)
  Qarray(31) = QloopLC(4)*Qarray(12)
  Qarray(32) = QloopLC(3)*Qarray(13)
  Qarray(33) = QloopLC(4)*Qarray(13)
  Qarray(34) = QloopLC(4)*Qarray(14)
  Qarray(35) = QloopLC(4)*Qarray(15)
  Qarray(36) = QloopLC(1)*Qarray(16)
  Qarray(37) = QloopLC(2)*Qarray(16)
  Qarray(38) = QloopLC(3)*Qarray(16)
  Qarray(39) = QloopLC(4)*Qarray(16)
  Qarray(40) = QloopLC(2)*Qarray(17)
  Qarray(41) = QloopLC(3)*Qarray(17)
  Qarray(42) = QloopLC(4)*Qarray(17)
  Qarray(43) = QloopLC(3)*Qarray(18)
  Qarray(44) = QloopLC(4)*Qarray(18)
  Qarray(45) = QloopLC(4)*Qarray(19)
  Qarray(46) = QloopLC(2)*Qarray(20)
  Qarray(47) = QloopLC(3)*Qarray(20)
  Qarray(48) = QloopLC(4)*Qarray(20)
  Qarray(49) = QloopLC(3)*Qarray(21)
  Qarray(50) = QloopLC(4)*Qarray(21)
  Qarray(51) = QloopLC(4)*Qarray(22)
  Qarray(52) = QloopLC(3)*Qarray(23)
  Qarray(53) = QloopLC(4)*Qarray(23)
  Qarray(54) = QloopLC(4)*Qarray(24)
  Qarray(55) = QloopLC(4)*Qarray(25)
  Qarray(56) = QloopLC(2)*Qarray(26)
  Qarray(57) = QloopLC(3)*Qarray(26)
  Qarray(58) = QloopLC(4)*Qarray(26)
  Qarray(59) = QloopLC(3)*Qarray(27)
  Qarray(60) = QloopLC(4)*Qarray(27)
  Qarray(61) = QloopLC(4)*Qarray(28)
  Qarray(62) = QloopLC(3)*Qarray(29)
  Qarray(63) = QloopLC(4)*Qarray(29)
  Qarray(64) = QloopLC(4)*Qarray(30)
  Qarray(65) = QloopLC(4)*Qarray(31)
  Qarray(66) = QloopLC(3)*Qarray(32)
  Qarray(67) = QloopLC(4)*Qarray(32)
  Qarray(68) = QloopLC(4)*Qarray(33)
  Qarray(69) = QloopLC(4)*Qarray(34)
  Qarray(70) = QloopLC(4)*Qarray(35)
  Qarray(71) = QloopLC(1)*Qarray(36)
  Qarray(72) = QloopLC(2)*Qarray(36)
  Qarray(73) = QloopLC(3)*Qarray(36)
  Qarray(74) = QloopLC(4)*Qarray(36)
  Qarray(75) = QloopLC(2)*Qarray(37)
  Qarray(76) = QloopLC(3)*Qarray(37)
  Qarray(77) = QloopLC(4)*Qarray(37)
  Qarray(78) = QloopLC(3)*Qarray(38)
  Qarray(79) = QloopLC(4)*Qarray(38)
  Qarray(80) = QloopLC(4)*Qarray(39)
  Qarray(81) = QloopLC(2)*Qarray(40)
  Qarray(82) = QloopLC(3)*Qarray(40)
  Qarray(83) = QloopLC(4)*Qarray(40)
  Qarray(84) = QloopLC(3)*Qarray(41)
  Qarray(85) = QloopLC(4)*Qarray(41)
  Qarray(86) = QloopLC(4)*Qarray(42)
  Qarray(87) = QloopLC(3)*Qarray(43)
  Qarray(88) = QloopLC(4)*Qarray(43)
  Qarray(89) = QloopLC(4)*Qarray(44)
  Qarray(90) = QloopLC(4)*Qarray(45)
  Qarray(91) = QloopLC(2)*Qarray(46)
  Qarray(92) = QloopLC(3)*Qarray(46)
  Qarray(93) = QloopLC(4)*Qarray(46)
  Qarray(94) = QloopLC(3)*Qarray(47)
  Qarray(95) = QloopLC(4)*Qarray(47)
  Qarray(96) = QloopLC(4)*Qarray(48)
  Qarray(97) = QloopLC(3)*Qarray(49)
  Qarray(98) = QloopLC(4)*Qarray(49)
  Qarray(99) = QloopLC(4)*Qarray(50)
  Qarray(100) = QloopLC(4)*Qarray(51)
  Qarray(101) = QloopLC(3)*Qarray(52)
  Qarray(102) = QloopLC(4)*Qarray(52)
  Qarray(103) = QloopLC(4)*Qarray(53)
  Qarray(104) = QloopLC(4)*Qarray(54)
  Qarray(105) = QloopLC(4)*Qarray(55)
  Qarray(106) = QloopLC(2)*Qarray(56)
  Qarray(107) = QloopLC(3)*Qarray(56)
  Qarray(108) = QloopLC(4)*Qarray(56)
  Qarray(109) = QloopLC(3)*Qarray(57)
  Qarray(110) = QloopLC(4)*Qarray(57)
  Qarray(111) = QloopLC(4)*Qarray(58)
  Qarray(112) = QloopLC(3)*Qarray(59)
  Qarray(113) = QloopLC(4)*Qarray(59)
  Qarray(114) = QloopLC(4)*Qarray(60)
  Qarray(115) = QloopLC(4)*Qarray(61)
  Qarray(116) = QloopLC(3)*Qarray(62)
  Qarray(117) = QloopLC(4)*Qarray(62)
  Qarray(118) = QloopLC(4)*Qarray(63)
  Qarray(119) = QloopLC(4)*Qarray(64)
  Qarray(120) = QloopLC(4)*Qarray(65)
  Qarray(121) = QloopLC(3)*Qarray(66)
  Qarray(122) = QloopLC(4)*Qarray(66)
  Qarray(123) = QloopLC(4)*Qarray(67)
  Qarray(124) = QloopLC(4)*Qarray(68)
  Qarray(125) = QloopLC(4)*Qarray(69)
  Qarray(126) = QloopLC(4)*Qarray(70)

end subroutine loop_mom_tens_5


!*********************************************
subroutine loop_mom_tens_6(QloopLC,Qarray)
  use KIND_TYPES, only: REALKIND
  implicit none
  complex(REALKIND), intent(in)  :: QloopLC(4)
  complex(REALKIND), intent(out) :: Qarray(210)

  Qarray(1) = 1
  Qarray(2) = QloopLC(1)*Qarray(1)
  Qarray(3) = QloopLC(2)*Qarray(1)
  Qarray(4) = QloopLC(3)*Qarray(1)
  Qarray(5) = QloopLC(4)*Qarray(1)
  Qarray(6) = QloopLC(1)*Qarray(2)
  Qarray(7) = QloopLC(2)*Qarray(2)
  Qarray(8) = QloopLC(3)*Qarray(2)
  Qarray(9) = QloopLC(4)*Qarray(2)
  Qarray(10) = QloopLC(2)*Qarray(3)
  Qarray(11) = QloopLC(3)*Qarray(3)
  Qarray(12) = QloopLC(4)*Qarray(3)
  Qarray(13) = QloopLC(3)*Qarray(4)
  Qarray(14) = QloopLC(4)*Qarray(4)
  Qarray(15) = QloopLC(4)*Qarray(5)
  Qarray(16) = QloopLC(1)*Qarray(6)
  Qarray(17) = QloopLC(2)*Qarray(6)
  Qarray(18) = QloopLC(3)*Qarray(6)
  Qarray(19) = QloopLC(4)*Qarray(6)
  Qarray(20) = QloopLC(2)*Qarray(7)
  Qarray(21) = QloopLC(3)*Qarray(7)
  Qarray(22) = QloopLC(4)*Qarray(7)
  Qarray(23) = QloopLC(3)*Qarray(8)
  Qarray(24) = QloopLC(4)*Qarray(8)
  Qarray(25) = QloopLC(4)*Qarray(9)
  Qarray(26) = QloopLC(2)*Qarray(10)
  Qarray(27) = QloopLC(3)*Qarray(10)
  Qarray(28) = QloopLC(4)*Qarray(10)
  Qarray(29) = QloopLC(3)*Qarray(11)
  Qarray(30) = QloopLC(4)*Qarray(11)
  Qarray(31) = QloopLC(4)*Qarray(12)
  Qarray(32) = QloopLC(3)*Qarray(13)
  Qarray(33) = QloopLC(4)*Qarray(13)
  Qarray(34) = QloopLC(4)*Qarray(14)
  Qarray(35) = QloopLC(4)*Qarray(15)
  Qarray(36) = QloopLC(1)*Qarray(16)
  Qarray(37) = QloopLC(2)*Qarray(16)
  Qarray(38) = QloopLC(3)*Qarray(16)
  Qarray(39) = QloopLC(4)*Qarray(16)
  Qarray(40) = QloopLC(2)*Qarray(17)
  Qarray(41) = QloopLC(3)*Qarray(17)
  Qarray(42) = QloopLC(4)*Qarray(17)
  Qarray(43) = QloopLC(3)*Qarray(18)
  Qarray(44) = QloopLC(4)*Qarray(18)
  Qarray(45) = QloopLC(4)*Qarray(19)
  Qarray(46) = QloopLC(2)*Qarray(20)
  Qarray(47) = QloopLC(3)*Qarray(20)
  Qarray(48) = QloopLC(4)*Qarray(20)
  Qarray(49) = QloopLC(3)*Qarray(21)
  Qarray(50) = QloopLC(4)*Qarray(21)
  Qarray(51) = QloopLC(4)*Qarray(22)
  Qarray(52) = QloopLC(3)*Qarray(23)
  Qarray(53) = QloopLC(4)*Qarray(23)
  Qarray(54) = QloopLC(4)*Qarray(24)
  Qarray(55) = QloopLC(4)*Qarray(25)
  Qarray(56) = QloopLC(2)*Qarray(26)
  Qarray(57) = QloopLC(3)*Qarray(26)
  Qarray(58) = QloopLC(4)*Qarray(26)
  Qarray(59) = QloopLC(3)*Qarray(27)
  Qarray(60) = QloopLC(4)*Qarray(27)
  Qarray(61) = QloopLC(4)*Qarray(28)
  Qarray(62) = QloopLC(3)*Qarray(29)
  Qarray(63) = QloopLC(4)*Qarray(29)
  Qarray(64) = QloopLC(4)*Qarray(30)
  Qarray(65) = QloopLC(4)*Qarray(31)
  Qarray(66) = QloopLC(3)*Qarray(32)
  Qarray(67) = QloopLC(4)*Qarray(32)
  Qarray(68) = QloopLC(4)*Qarray(33)
  Qarray(69) = QloopLC(4)*Qarray(34)
  Qarray(70) = QloopLC(4)*Qarray(35)
  Qarray(71) = QloopLC(1)*Qarray(36)
  Qarray(72) = QloopLC(2)*Qarray(36)
  Qarray(73) = QloopLC(3)*Qarray(36)
  Qarray(74) = QloopLC(4)*Qarray(36)
  Qarray(75) = QloopLC(2)*Qarray(37)
  Qarray(76) = QloopLC(3)*Qarray(37)
  Qarray(77) = QloopLC(4)*Qarray(37)
  Qarray(78) = QloopLC(3)*Qarray(38)
  Qarray(79) = QloopLC(4)*Qarray(38)
  Qarray(80) = QloopLC(4)*Qarray(39)
  Qarray(81) = QloopLC(2)*Qarray(40)
  Qarray(82) = QloopLC(3)*Qarray(40)
  Qarray(83) = QloopLC(4)*Qarray(40)
  Qarray(84) = QloopLC(3)*Qarray(41)
  Qarray(85) = QloopLC(4)*Qarray(41)
  Qarray(86) = QloopLC(4)*Qarray(42)
  Qarray(87) = QloopLC(3)*Qarray(43)
  Qarray(88) = QloopLC(4)*Qarray(43)
  Qarray(89) = QloopLC(4)*Qarray(44)
  Qarray(90) = QloopLC(4)*Qarray(45)
  Qarray(91) = QloopLC(2)*Qarray(46)
  Qarray(92) = QloopLC(3)*Qarray(46)
  Qarray(93) = QloopLC(4)*Qarray(46)
  Qarray(94) = QloopLC(3)*Qarray(47)
  Qarray(95) = QloopLC(4)*Qarray(47)
  Qarray(96) = QloopLC(4)*Qarray(48)
  Qarray(97) = QloopLC(3)*Qarray(49)
  Qarray(98) = QloopLC(4)*Qarray(49)
  Qarray(99) = QloopLC(4)*Qarray(50)
  Qarray(100) = QloopLC(4)*Qarray(51)
  Qarray(101) = QloopLC(3)*Qarray(52)
  Qarray(102) = QloopLC(4)*Qarray(52)
  Qarray(103) = QloopLC(4)*Qarray(53)
  Qarray(104) = QloopLC(4)*Qarray(54)
  Qarray(105) = QloopLC(4)*Qarray(55)
  Qarray(106) = QloopLC(2)*Qarray(56)
  Qarray(107) = QloopLC(3)*Qarray(56)
  Qarray(108) = QloopLC(4)*Qarray(56)
  Qarray(109) = QloopLC(3)*Qarray(57)
  Qarray(110) = QloopLC(4)*Qarray(57)
  Qarray(111) = QloopLC(4)*Qarray(58)
  Qarray(112) = QloopLC(3)*Qarray(59)
  Qarray(113) = QloopLC(4)*Qarray(59)
  Qarray(114) = QloopLC(4)*Qarray(60)
  Qarray(115) = QloopLC(4)*Qarray(61)
  Qarray(116) = QloopLC(3)*Qarray(62)
  Qarray(117) = QloopLC(4)*Qarray(62)
  Qarray(118) = QloopLC(4)*Qarray(63)
  Qarray(119) = QloopLC(4)*Qarray(64)
  Qarray(120) = QloopLC(4)*Qarray(65)
  Qarray(121) = QloopLC(3)*Qarray(66)
  Qarray(122) = QloopLC(4)*Qarray(66)
  Qarray(123) = QloopLC(4)*Qarray(67)
  Qarray(124) = QloopLC(4)*Qarray(68)
  Qarray(125) = QloopLC(4)*Qarray(69)
  Qarray(126) = QloopLC(4)*Qarray(70)
  Qarray(127) = QloopLC(1)*Qarray(71)
  Qarray(128) = QloopLC(2)*Qarray(71)
  Qarray(129) = QloopLC(3)*Qarray(71)
  Qarray(130) = QloopLC(4)*Qarray(71)
  Qarray(131) = QloopLC(2)*Qarray(72)
  Qarray(132) = QloopLC(3)*Qarray(72)
  Qarray(133) = QloopLC(4)*Qarray(72)
  Qarray(134) = QloopLC(3)*Qarray(73)
  Qarray(135) = QloopLC(4)*Qarray(73)
  Qarray(136) = QloopLC(4)*Qarray(74)
  Qarray(137) = QloopLC(2)*Qarray(75)
  Qarray(138) = QloopLC(3)*Qarray(75)
  Qarray(139) = QloopLC(4)*Qarray(75)
  Qarray(140) = QloopLC(3)*Qarray(76)
  Qarray(141) = QloopLC(4)*Qarray(76)
  Qarray(142) = QloopLC(4)*Qarray(77)
  Qarray(143) = QloopLC(3)*Qarray(78)
  Qarray(144) = QloopLC(4)*Qarray(78)
  Qarray(145) = QloopLC(4)*Qarray(79)
  Qarray(146) = QloopLC(4)*Qarray(80)
  Qarray(147) = QloopLC(2)*Qarray(81)
  Qarray(148) = QloopLC(3)*Qarray(81)
  Qarray(149) = QloopLC(4)*Qarray(81)
  Qarray(150) = QloopLC(3)*Qarray(82)
  Qarray(151) = QloopLC(4)*Qarray(82)
  Qarray(152) = QloopLC(4)*Qarray(83)
  Qarray(153) = QloopLC(3)*Qarray(84)
  Qarray(154) = QloopLC(4)*Qarray(84)
  Qarray(155) = QloopLC(4)*Qarray(85)
  Qarray(156) = QloopLC(4)*Qarray(86)
  Qarray(157) = QloopLC(3)*Qarray(87)
  Qarray(158) = QloopLC(4)*Qarray(87)
  Qarray(159) = QloopLC(4)*Qarray(88)
  Qarray(160) = QloopLC(4)*Qarray(89)
  Qarray(161) = QloopLC(4)*Qarray(90)
  Qarray(162) = QloopLC(2)*Qarray(91)
  Qarray(163) = QloopLC(3)*Qarray(91)
  Qarray(164) = QloopLC(4)*Qarray(91)
  Qarray(165) = QloopLC(3)*Qarray(92)
  Qarray(166) = QloopLC(4)*Qarray(92)
  Qarray(167) = QloopLC(4)*Qarray(93)
  Qarray(168) = QloopLC(3)*Qarray(94)
  Qarray(169) = QloopLC(4)*Qarray(94)
  Qarray(170) = QloopLC(4)*Qarray(95)
  Qarray(171) = QloopLC(4)*Qarray(96)
  Qarray(172) = QloopLC(3)*Qarray(97)
  Qarray(173) = QloopLC(4)*Qarray(97)
  Qarray(174) = QloopLC(4)*Qarray(98)
  Qarray(175) = QloopLC(4)*Qarray(99)
  Qarray(176) = QloopLC(4)*Qarray(100)
  Qarray(177) = QloopLC(3)*Qarray(101)
  Qarray(178) = QloopLC(4)*Qarray(101)
  Qarray(179) = QloopLC(4)*Qarray(102)
  Qarray(180) = QloopLC(4)*Qarray(103)
  Qarray(181) = QloopLC(4)*Qarray(104)
  Qarray(182) = QloopLC(4)*Qarray(105)
  Qarray(183) = QloopLC(2)*Qarray(106)
  Qarray(184) = QloopLC(3)*Qarray(106)
  Qarray(185) = QloopLC(4)*Qarray(106)
  Qarray(186) = QloopLC(3)*Qarray(107)
  Qarray(187) = QloopLC(4)*Qarray(107)
  Qarray(188) = QloopLC(4)*Qarray(108)
  Qarray(189) = QloopLC(3)*Qarray(109)
  Qarray(190) = QloopLC(4)*Qarray(109)
  Qarray(191) = QloopLC(4)*Qarray(110)
  Qarray(192) = QloopLC(4)*Qarray(111)
  Qarray(193) = QloopLC(3)*Qarray(112)
  Qarray(194) = QloopLC(4)*Qarray(112)
  Qarray(195) = QloopLC(4)*Qarray(113)
  Qarray(196) = QloopLC(4)*Qarray(114)
  Qarray(197) = QloopLC(4)*Qarray(115)
  Qarray(198) = QloopLC(3)*Qarray(116)
  Qarray(199) = QloopLC(4)*Qarray(116)
  Qarray(200) = QloopLC(4)*Qarray(117)
  Qarray(201) = QloopLC(4)*Qarray(118)
  Qarray(202) = QloopLC(4)*Qarray(119)
  Qarray(203) = QloopLC(4)*Qarray(120)
  Qarray(204) = QloopLC(3)*Qarray(121)
  Qarray(205) = QloopLC(4)*Qarray(121)
  Qarray(206) = QloopLC(4)*Qarray(122)
  Qarray(207) = QloopLC(4)*Qarray(123)
  Qarray(208) = QloopLC(4)*Qarray(124)
  Qarray(209) = QloopLC(4)*Qarray(125)
  Qarray(210) = QloopLC(4)*Qarray(126)

end subroutine loop_mom_tens_6

end module ol_loop_momentum_/**/REALKIND
